# ZCons
#
# Zimmermann's SCons wrapper
#
# Copyright (C) 2016-2018 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# ZCons is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# ZCons is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with ZCons. If not, see <http://www.gnu.org/licenses/>.

"""
ZCons exception classes.

.. moduleauthor:: Stefan Zimmermann <zimmermann.code@gmail.com>
"""

__all__ = ('ZConsError', 'ZConsResolveSConsError', 'ZConsSConsError')


class ZConsError(RuntimeError):
    """Base class for zcons exceptions."""


class ZConsResolveSConsError(ZConsError):
    """
    SCons could not be resolved on ``import zcons``.

    Which means that either an existing SCons installation is corrupted or the
    on-demand installation failed
    """

    def __init__(self, reason=None):
        """Give an optional `reason` as either string or exception object."""
        msg = "Failed to resolve SCons installation"
        if reason is not None:
            if isinstance(reason, Exception):
                msg += " (%s: %s)" % (type(reason).__name__, reason)
            else:
                msg += " (%s)" % reason
        super(ZConsResolveSConsError, self).__init__(msg)


class ZConsSConsError(ZConsError):
    """Running SCons via :func:`zetup.scons` failed."""

    def __init__(self, args=None):
        """Give optional `args` that where used for running SCons."""
        msg = "Failed to run SCons"
        if args is not None:
            msg += " with {!r}".format(list(args))
        super(ZConsSConsError, self).__init__(msg)



# ZCons
#
# Zimmermann's SCons wrapper
#
# Copyright (C) 2016 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# ZCons is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# ZCons is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with ZCons. If not, see <http://www.gnu.org/licenses/>.

"""
Zimmermann's SCons wrapper.

Checks availability of SCons and, if not found, implicitly installs SCons as
local Python egg.

Defines :func:`zetup.scons` for running SCons sub-processes

.. moduleauthor:: Stefan Zimmermann <zimmermann.code@gmail.com>
"""
import sys

import zetup
from zetup.version import Version
from path import Path
from pkg_resources import get_distribution

from .error import ZConsError, ZConsSConsError, ZConsResolveSConsError

zetup.toplevel(__name__, [
    'Environment',
    'ZConsError', 'ZConsSConsError', 'ZConsResolveSConsError',
    'scons',
])
# __call__=lambda *args, **options: scons(*args, **options)


REQUIRES_SCONS = "SCons >=  3.0.0"

zetup.resolve([REQUIRES_SCONS])

SCONS = get_distribution(REQUIRES_SCONS)

SCONS_ROOT = Path(SCONS.location) / ("scons")
if Version(SCONS.version) < '3.0.3':
    SCONS_ROOT += "-%s" % SCONS.version

if not SCONS_ROOT.exists():
    raise ZConsResolveSConsError(reason=(
        "SCons package root does not exist: {}".format(SCONS_ROOT)))

sys.path.insert(0, str(SCONS_ROOT))

import SCons  # pylint: disable=import-error

if SCons.__version__ != SCONS.version:
    raise ZConsResolveSConsError(reason=(
        "SCons.__version__ {} doesn't match {!r}"
        .format(SCons.__version__, SCONS)))

print("SCons package root: {}".format(SCONS_ROOT))

from SCons.Environment import Environment  # pylint: disable=import-error


def scons(*args, **options):
    """
    Run SCons as separate sub-process through ``sys.executable``.

    :param args:
        SCons command line arguments
    :param options:
        SCons variables and keyword arguments for ``zetup.call``

    All `options` starting with upper case letters are used as SCons variables
    and appended to `args`

    See ``zetup.call`` and ``subprocess.call`` for info about general
    `options`
    """
    args = list(args)
    for name in list(options):
        if name[0].isupper():
            args.append('%s=%s' % (name, options.pop(name)))

    if zetup.call([
            sys.executable, '-c', "__import__('SCons.Script').Script.main()"
    ] + args, **options):
        raise ZConsSConsError(args=args)


def scons_debug(*args, **options):
    """
    Run SCons with ``--debug=pdb``.

    See :func:`zcons.scons` for info about arguments and the sub-process
    """
    args = list(args)
    for name in list(options):
        if name[0].isupper():
            args.append('%s=%s' % (name, options.pop(name)))

    if zetup.call([
            sys.executable, '-c',
            "__import__('SCons.Script').Script.main()",
            "--debug=pdb"
    ] + args, **options):
        raise ZConsSConsError(args=args)


scons.debug = scons_debug



# ZCons
#
# Zimmermann's SCons wrapper
#
# Copyright (C) 2016 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# ZCons is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# ZCons is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with ZCons. If not, see <http://www.gnu.org/licenses/>.

"""zcons.__main__

Just runs the entry point of ``zcons`` command line script.

.. moduleauthor:: Stefan Zimmermann <zimmermann.code@gmail.com>
"""

import zcons.script


zcons.script.run()



# This file was auto-generated by ZETUP
#
# Zimmermann's Extensible Tools for Unified Project setups
#
# https://github.com/zimmermanncode/zetup

from __future__ import print_function

from setuptools import setup


dist = None
try:
    dist = setup(
        setup_requires=["zetup >= 0.2.60"],

        use_zetup=True,
    )
finally:
    if dist is not None and hasattr(dist, 'zetup_made'):
        dist.zetup_made.clean()


# ZCons
#
# Zimmermann's SCons wrapper
#
# Copyright (C) 2016 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# ZCons is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# ZCons is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with ZCons. If not, see <http://www.gnu.org/licenses/>.

"""zcons.script

The zcons command line script.

.. moduleauthor:: Stefan Zimmermann <zimmermann.code@gmail.com>
"""
import sys

import zcons


def run():
    """Entry point for ``zcons`` command line script.

    - Calls :func:`zcons.scons` with ``sys.argv[1:]`` as arguments.
    """
    zcons.scons(*sys.argv[1:])


if __name__ == '__main__':
    run()
